=== modified file 'ivle/database.py'
--- ivle/database.py	2009-12-08 11:35:25 +0000
+++ ivle/database.py	2009-12-08 11:41:44 +0000
@@ -375,9 +375,14 @@
             if (enrolment and enrolment.role in (u'tutor', u'lecturer')) \
                or user.admin:
                 perms.add('edit')
-                # XXX Bug #493945 -- should tutors have this permission?
+                # XXX Bug #493945 -- should tutors have these permissions?
                 # Potentially move into the next category (lecturer & admin)
-                perms.add('enrol')
+                perms.add('enrol')          # Can see enrolment screen at all
+                perms.add('enrol_student')  # Can enrol students
+            if (enrolment and enrolment.role in (u'lecturer')) or user.admin:
+                perms.add('enrol_tutor')    # Can enrol tutors
+            if user.admin:
+                perms.add('enrol_lecturer') # Can enrol lecturers
         return perms
 
     def get_enrolment(self, user):

=== modified file 'ivle/webapp/admin/subject.py'
--- ivle/webapp/admin/subject.py	2009-12-08 11:35:25 +0000
+++ ivle/webapp/admin/subject.py	2009-12-08 11:41:44 +0000
@@ -100,8 +100,25 @@
         return value
 
 
+class RoleEnrolmentValidator(formencode.FancyValidator):
+    """A FormEncode validator that checks permission to enrol users with a
+    particular role.
+
+    The state must have an 'offering' attribute.
+    """
+    def _to_python(self, value, state):
+        if ("enrol_" + value) not in state.offering.get_permissions(state.user):
+            raise formencode.Invalid('Not allowed to assign users that role',
+                                     value, state)
+        return value
+
+
 class EnrolSchema(formencode.Schema):
     user = formencode.All(NoEnrolmentValidator(), UserValidator())
+    role = formencode.All(formencode.validators.OneOf(
+                                ["lecturer", "tutor", "student"]),
+                          RoleEnrolmentValidator(),
+                          formencode.validators.UnicodeString())
 
 
 class EnrolmentsView(XHTMLView):
@@ -128,7 +145,7 @@
                 validator = EnrolSchema()
                 req.offering = self.context # XXX: Getting into state.
                 data = validator.to_python(data, state=req)
-                self.context.enrol(data['user'])
+                self.context.enrol(data['user'], data['role'])
                 req.store.commit()
                 req.throw_redirect(req.uri)
             except formencode.Invalid, e:

=== modified file 'ivle/webapp/admin/templates/enrol.html'
--- ivle/webapp/admin/templates/enrol.html	2009-05-03 07:08:10 +0000
+++ ivle/webapp/admin/templates/enrol.html	2009-12-08 11:41:44 +0000
@@ -10,7 +10,14 @@
         <p>
           <label for="user">Username:</label>
           <input type="text" name="user" />
+          <label for="role">Enrol as:</label>
+          <select name="role">
+            <option value="student" selected="selected">Student</option>
+            <option value="tutor">Tutor</option>
+            <option value="lecturer">Lecturer</option>
+          </select>
           <span py:if="'user' in errors" class="form_error">${errors.user}</span>
+          <span py:if="'role' in errors" class="form_error">${errors.role}</span>
           <br />
           <input type="submit" value="Enrol" />
         </p>

